@@ -21,8 +21,9 @@ import (
 	"crypto/tls"
 	"time"
 
-	"github.com/apache/pulsar-client-go/pulsar/internal/auth"
+	"github.com/apache/pulsar-client-go/pulsar/auth"
 	"github.com/apache/pulsar-client-go/pulsar/log"
+	"github.com/prometheus/client_golang/prometheus"
 )
 
 // NewClient Creates a pulsar client instance
@@ -78,6 +79,11 @@ func NewAuthenticationOAuth2(authParams map[string]string) Authentication {
 	return oauth
 }
 
+// NewAuthenticationBasic Creates Basic Authentication provider
+func NewAuthenticationBasic(username, password string) (Authentication, error) {
+	return auth.NewAuthenticationBasic(username, password)
+}
+
 // ClientOptions is used to construct a Pulsar Client instance.
 type ClientOptions struct {
 	// Configure the service URL for the Pulsar service.
@@ -92,10 +98,19 @@ type ClientOptions struct {
 	// operation will be marked as failed
 	OperationTimeout time.Duration
 
+	// Configure the ping send and check interval, default to 30 seconds.
+	KeepAliveInterval time.Duration
+
 	// Configure the authentication provider. (default: no authentication)
 	// Example: `Authentication: NewAuthenticationTLS("my-cert.pem", "my-key.pem")`
 	Authentication
 
+	// Set the path to the TLS key file
+	TLSKeyFilePath string
+
+	// Set the path to the TLS certificate file
+	TLSCertificateFile string
+
 	// Set the path to the trusted TLS certificate file
 	TLSTrustCertsFilePath string
 
@@ -105,6 +120,15 @@ type ClientOptions struct {
 	// Configure whether the Pulsar client verify the validity of the host name from broker (default: false)
 	TLSValidateHostname bool
 
+	// TLSCipherSuites is a list of enabled TLS 1.0–1.2 cipher suites. See tls.Config CipherSuites for more information.
+	TLSCipherSuites []uint16
+
+	// TLSMinVersion contains the minimum TLS version that is acceptable. See tls.Config MinVersion for more information.
+	TLSMinVersion uint16
+
+	// TLSMaxVersion contains the maximum TLS version that is acceptable. See tls.Config MaxVersion for more information.
+	TLSMaxVersion uint16
+
 	// Configure the net model for vpc user to connect the pulsar broker
 	ListenerName string
 
@@ -117,8 +141,26 @@ type ClientOptions struct {
 	// FIXME: use `logger` as internal field name instead of `log` as it's more idiomatic
 	Logger log.Logger
 
+	// Specify metric cardinality to the tenant, namespace or topic levels, or remove it completely.
+	// Default: MetricsCardinalityNamespace
+	MetricsCardinality MetricsCardinality
+
 	// Add custom labels to all the metrics reported by this client instance
 	CustomMetricsLabels map[string]string
+
+	// Specify metric registerer used to register metrics.
+	// Default prometheus.DefaultRegisterer
+	MetricsRegisterer prometheus.Registerer
+
+	// Release the connection if it is not used for more than ConnectionMaxIdleTime.
+	// Default is 180 seconds, minimum is 60 seconds. Negative such as -1 to disable.
+	ConnectionMaxIdleTime time.Duration
+
+	EnableTransaction bool
+
+	// Limit of client memory usage (in byte). The 64M default can guarantee a high producer throughput.
+	// Config less than 0 indicates off memory limit.
+	MemoryLimitBytes int64
 }
 
 // Client represents a pulsar client
@@ -137,6 +179,10 @@ type Client interface {
 	// This method will block until the reader is created successfully.
 	CreateReader(ReaderOptions) (Reader, error)
 
+	// CreateTableView creates a table view instance.
+	// This method will block until the table view is created successfully.
+	CreateTableView(TableViewOptions) (TableView, error)
+
 	// TopicPartitions Fetches the list of partitions for a given topic
 	//
 	// If the topic is partitioned, this will return a list of partition names.
@@ -147,6 +193,28 @@ type Client interface {
 	// {@link Consumer} or {@link Producer} instances directly on a particular partition.
 	TopicPartitions(topic string) ([]string, error)
 
+	// NewTransaction creates a new Transaction instance.
+	//
+	// This function is used to initiate a new transaction for performing
+	// atomic operations on the message broker. It returns a Transaction
+	// object that can be used to produce, consume and commit messages in a
+	// transactional manner.
+	//
+	// In case of any errors while creating the transaction, an error will
+	// be returned.
+	NewTransaction(duration time.Duration) (Transaction, error)
+
 	// Close Closes the Client and free associated resources
 	Close()
 }
+
+// MetricsCardinality represents the specificty of labels on a per-metric basis
+type MetricsCardinality int
+
+const (
+	_                           MetricsCardinality = iota
+	MetricsCardinalityNone                         // Do not add additional labels to metrics
+	MetricsCardinalityTenant                       // Label metrics by tenant
+	MetricsCardinalityNamespace                    // Label metrics by tenant and namespace
+	MetricsCardinalityTopic                        // Label metrics by topic
+)
